
// C++ conversion of the java/PlotExample.java example.

#include <AIDA/AIDA.h>

#include "Random.h"

using namespace AIDA;

#include <stdlib.h>

int main(int argc,char* argv[]) {

  IAnalysisFactory* af = AIDA_createAnalysisFactory();
  if(!af) return EXIT_FAILURE;

  ITreeFactory* trf = af->createTreeFactory();
  if(!trf) return EXIT_FAILURE;
  ITree* tree = trf->create();
  delete trf;
  if(!tree) return EXIT_FAILURE;

  IHistogramFactory* hf = af->createHistogramFactory(*tree);
  if(!hf) return EXIT_FAILURE;
  IHistogram2D* h2d = hf->createHistogram2D("test 2d",50,-30,30,50,-3,3);
  if(!h2d) return EXIT_FAILURE;
      
  Random r;
  for (int i=0; i<10000; i++) {
    h2d->fill(10*r.nextGaussian(),r.nextGaussian());
  }
      
  IPlotterFactory* pf = af->createPlotterFactory(argc,argv);
  if(!pf) return EXIT_FAILURE;
  IPlotter* plotter = pf->create();
  delete pf;
  if(!plotter) return EXIT_FAILURE;

  plotter->destroyRegions();
  plotter->createRegion(0,0,.66,1)->plot(*h2d);
  plotter->createRegion(.66,0,.33,.5)->plot(*hf->projectionX("X Projection",*h2d));
  plotter->createRegion(.66,.5,.33,.5)->plot(*hf->projectionY("Y Projection",*h2d));
  plotter->show();
  plotter->interact();

  delete hf;

  delete plotter;

  delete af;

  return EXIT_SUCCESS;
}
